#!/usr/bin/env python3
# encoding: utf-8
#
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2024 Indiscale GmbH <info@indiscale.com>
# Copyright (C) 2024 Florian Spreckelsen <f.spreckelsen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
import json
import logging
import sys

import linkahead as db
from caosadvancedtools.serverside import helper
from caosadvancedtools.serverside.logging import configure_server_side_logging
from linkahead.utils.escape import escape_squoted_text

logger = logging.getLogger("caosadvancedtools")


def main():

    # We only need the default arguments for SSS
    parser = helper.get_argument_parser()
    args = parser.parse_args()

    # We assume that if there is an authtoken, we're run serverside
    if hasattr(args, "auth_token") and args.auth_token:
        db.configure_connection(auth_token=args.auth_token)
        debug_file = configure_server_side_logging()
    else:
        # logging for local execution, print info on stdout
        logger.addHandler(logging.StreamHandler(sys.stdout))
        logger.setLevel(logging.DEBUG)
        debug_file = None

    if hasattr(args, "filename") and args.filename:
        with open(args.filename) as form_json:
            form_data = json.load(form_json)
    else:
        logger.error("Sample-template registration didn't receive any form information.")
        return 0

    template_rt = db.get_entity_by_name("RegisterSampleTemplate", role="RECORDTYPE")
    template_name = form_data["template_name"]
    candidates = db.execute_query(
        f"FIND '{template_rt.name}' WITH name='{escape_squoted_text(template_name)}'"
    )
    if candidates:
        logger.error(
            f"A {template_rt.name} with name '{template_name}' exists in the database already. "
            "Please choose a different name or rename the old entity."
        )
        return 0
    rec = db.Record(name=template_name).add_parent(template_rt)
    rec.add_property(name="column_names",
                     value=form_data["column_names"], datatype=db.LIST(db.TEXT))
    rec.add_property(name="Main User", value=form_data["main_user"])
    try:
        rec.insert()
    except db.TransactionError as te:
        logger.error(f"There was a problem inserting your {template_rt.name}:\n\n{te}")
        return 0
    logger.info(
        f"Your {template_rt.name} was inserted successfully. You can use it for sample registration "
        f"or visit it <a href=\"/Entity/{rec.id}\">here</a>."
    )
    return 0


if __name__ == "__main__":

    sys.exit(main())
